package	jp.co.sds_corp.fixconnect.message;

import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.text.DecimalFormat;

public class FIXMessage implements Serializable
{
	private static final long serialVersionUID = 1L;
	public static final byte SOH = 0x01;

	Node	header = new Node();
	Node	topNode = new Node();
	Node	trailer = new Node();

	public FIXMessage() {
	}

	public String getValue( Tag tag ) {
		return topNode.getValue( tag );
	}

	public String putValue( Tag tag, String value ) {
		return topNode.putValue( tag, value );
	}

	public String putValue( Field fld ) {
		return topNode.putValue( fld.getTag(), fld.getValue() );
	}

	public Component addComponent( Field field ) {
		return topNode.addComponent( field );
	}

	public Node getHeader() {
		return header;
	}

	public Node getTrailer() {
		return trailer;
	}

	public void getBytes( ByteArrayOutputStream bos, DictionaryLoader dict )
		throws Exception
	{
		Tag tag = dict.findTagName( "MsgType" );
		msgType = header.getValue( tag );
		if ( msgType == null ) {
			throw( new Exception( "MsgType not defined." ));
		}
		msgTypeName = dict.getMsgTypeName( msgType );

		ByteArrayOutputStream bodyBos = new ByteArrayOutputStream();
		header.getBytes( bodyBos, dict.getHeaderDictionary() );
		topNode.getBytes( bodyBos, dict.getMessageDictionary( msgType ) );
		trailer.getBytes( bodyBos, dict.getTrailerDictionary() );

		PrintWriter pw = new PrintWriter( bos );

		tag = dict.findTagName( "BeginString" );
		String ver = header.getValue( tag );
		pw.print( tag.getTagId() + "=" + ver );
		pw.flush();
		bos.write( FIXMessage.SOH );

		tag = dict.findTagName( "BodyLength" );
		bodyLength = String.valueOf( bodyBos.size() );
		pw.print( tag.getTagId() + "=" + bodyLength );
		pw.flush();
		bos.write( FIXMessage.SOH );

		bodyBos.writeTo( bos );

		checkSum = calcCheckSum( bos );
		pw.print( Tag.CheckSum.getTagId() + "=" + checkSum );
		pw.flush();
		pw.close();
		bos.write( FIXMessage.SOH );

		setLength( bos.size() );
	}

	String calcCheckSum( ByteArrayOutputStream bos ) {
		byte[] data = bos.toByteArray();
		long sum = 0;
		for ( int i = 0; i < data.length; i++ ) {
			sum += data[i];
		}
		DecimalFormat fmt = new DecimalFormat( "000" );
		String ret = fmt.format( sum % 256 );
		return ret;
	}

	public String toString() {
		String ret = getMsgTypeName() + " H:" + header.toString();
		ret += " B:" + topNode.toString();
		ret += " T:" + trailer.toString();
		ret += " bodyLength=" + bodyLength;
		ret += " checkSum=" + checkSum;
		ret += " length=" + getLength();
		return ret;
	}

	byte[]	buffer;	// バッファ
	int	length;	// 電文長
	String	msgType;	// MsgType
	String	msgTypeName;	// MsgTypeName
	String	bodyLength;	// BodyLength
	String	checkSum;	// CheckSum

	/**
	* バッファを設定します。
	* @param	var		設定するデータ
	*/
	public void setBuffer( byte[] var ) {
		this.buffer = var;
	}
	/**
	* バッファを取得します。
	* @return	バッファ
	*/
	public byte[] getBuffer() {
		return buffer;
	}

	/**
	* 電文長を設定します。
	* @param	var		設定するデータ
	*/
	public void setLength( int var ) {
		this.length = var;
	}
	/**
	* 電文長を取得します。
	* @return	電文長
	*/
	public int getLength() {
		return length;
	}

	/**
	* MsgTypeを設定します。
	* @param	var		設定するデータ
	*/
	public void setMsgType( String var ) {
		this.msgType = var;
	}
	/**
	* MsgTypeを取得します。
	* @return	MsgType
	*/
	public String getMsgType() {
		return msgType;
	}

	/**
	* MsgTypeNameを設定します。
	* @param	var		設定するデータ
	*/
	public void setMsgTypeName( String var ) {
		this.msgTypeName = var;
	}
	/**
	* MsgTypeNameを取得します。
	* @return	MsgTypeName
	*/
	public String getMsgTypeName() {
		return msgTypeName;
	}

	/**
	* BodyLengthを設定します。
	* @param	var		設定するデータ
	*/
	public void setBodyLength( String var ) {
		this.bodyLength = var;
	}
	/**
	* BodyLengthを取得します。
	* @return	BodyLength
	*/
	public String getBodyLength() {
		return bodyLength;
	}

	/**
	* CheckSumを設定します。
	* @param	var		設定するデータ
	*/
	public void setCheckSum( String var ) {
		this.checkSum = var;
	}
	/**
	* CheckSumを取得します。
	* @return	CheckSum
	*/
	public String getCheckSum() {
		return checkSum;
	}

}
