package	jp.co.sds_corp.fixconnect.transport;

import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import jp.co.sds_corp.fixconnect.FIXException;
import jp.co.sds_corp.fixconnect.logger.IMessageLogger;
import jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.fixconnect.message.FIXParser;
import jp.co.sds_corp.fixconnect.message.IMessageReceiver;
import jp.co.sds_corp.fixconnect.message.MessageReceiveEvent;
import jp.co.sds_corp.fixconnect.store.IMessageStore;
import jp.co.sds_corp.util.common.BlockQueue;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;

public class FIXSession implements IMessageReceiver, ITransportEventListener
{
	static CmnLogger logger = CmnLogger.getInstance();
	IMessageStore	store;
	Vector<IMessageLogger>	vecLogger = new Vector<IMessageLogger>();
	Vector<ITransportEventListener>	vecTrn = new Vector<ITransportEventListener>();
	Vector<IMessageReceiver>	vecRcv = new Vector<IMessageReceiver>();
	BlockQueue<FIXMessage>	que = new BlockQueue<FIXMessage>();

	public FIXSession() {
		recvBufSize = 10240;
		socketTimeout = 1000;
		timeout = 5000;
	}

	public FIXSession( String host, int port ) {
		this();
		socket = new FIXSocket( host, port );
		this.sessionId = String.valueOf( hashCode() );
	}

	public FIXSession( Socket sock ) {
		this();
		this.socket = new FIXSocket( sock );
		this.sessionId = String.valueOf( hashCode() );
	}

	public void init( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		String var;
		var = attr.get( "timeout", null );
		if ( var != null ) {
			timeout = Integer.valueOf( var ).intValue();
		}
		var = attr.get( "recvBufSize", null );
		if ( var != null ) {
			recvBufSize = Integer.valueOf( var ).intValue();
		}
		var = attr.get( "socketTimeout", null );
		if ( var != null ) {
			socketTimeout = Integer.valueOf( var ).intValue();
		}
		var = attr.get( "debug", "false" );
		if ( var.equals( "true" )) {
			setDebug( true );
		}

		IConfiguration c;
		c = config.getChild( "Parser" );
		parser = createParser( c );
		if ( socket != null ) {
			socket.setParser( parser );
		}
		c = config.getChild( "Store" );
		store = createStore( c );

		Iterator<IConfiguration> itr;
		itr = config.getChilds( "Logger" );
		while ( itr.hasNext()) {
			IConfiguration lc = itr.next();
			IMessageLogger logger = createLogger( lc );
			vecLogger.add( logger );
		}
	}

	FIXParser createParser( IConfiguration config ) throws Exception {
		FIXParser parser = FIXParser.createParser( config );
		return parser;
	}

	IMessageStore createStore( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		Class cls = Class.forName( attr.get( "class" ));
		store = (IMessageStore )cls.newInstance();
		store.init( config );
		return store;
	}

	IMessageLogger createLogger( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		Class cls = Class.forName( attr.get( "class" ));
		IMessageLogger logger = (IMessageLogger )cls.newInstance();
		logger.init( config );
		return logger;
	}

	public void start() throws Exception {
		socket.addMessageReceiver( this );
		socket.addEventListener( this );
		socket.start();
	}

	public void close() throws Exception {
		socket.close();
	}

	public void putValue( FIXMessage msg, String tagname, String value )
		throws Exception
	{
		getSocket().getParser().putValue( msg, tagname, value );
	}

	public void onEvent( TransportEvent ev ) {
		switch ( ev.getType()) {
		case CLOSE:
			socketClosed( ev.getSource() );
			break;
		}
		fireEventOccured( ev );
	}
	public void onException( FIXException ex ) {
		fireExceptionOccured( ex );
	}

	void socketClosed( FIXSocket socket ) {
		logger.info( this, "socket closed. "
			+ "session id=" + getSessionId() + " socket=[" + socket + "]" );
		try {
			socket.close();
		} catch ( Exception ex ) {
			logger.error( this, "socket close error", ex );
		}
	}

	public void addEventListener( ITransportEventListener l ) {
		vecTrn.add( l );
	}
	public boolean removeEventListener( ITransportEventListener l ) {
		return vecTrn.remove( l );
	}
	public void fireEventOccured( TransportEvent ev ) {
		ev.setSession( this );
		for ( int i = 0; i < vecTrn.size(); i++ ) {
			ITransportEventListener l = vecTrn.get( i );
			l.onEvent( ev );
		}
	}
	public void fireExceptionOccured( FIXException ex ) {
		for ( int i = 0; i < vecTrn.size(); i++ ) {
			ITransportEventListener l = vecTrn.get( i );
			l.onException( ex );
		}
	}

	public void addMessageReceiver( IMessageReceiver l ) {
		vecRcv.add( l );
	}
	public boolean removeMessageReceiver( IMessageReceiver l ) {
		return vecRcv.remove( l );
	}
	public void fireMessageReceived( MessageReceiveEvent ev ) {
		ev.setSession( this );
		for ( int i = 0; i < vecRcv.size(); i++ ) {
			IMessageReceiver l = vecRcv.get( i );
			l.onMessage( ev );
		}
	}

	public void onMessage( MessageReceiveEvent ev ) {
		try {
			FIXMessage msg = ev.getMessage();
			getSocket().getParser().validate( msg );
			que.add( msg );
			log( IMessageLogger.Direction.RECV, msg );
			String seq = parser.getHeader( msg, "MsgSeqNum" );
			store.setRecvSeqNum( Integer.valueOf( seq ) );
			fireMessageReceived( ev );
		} catch ( Exception ex ) {
			logger.error( this, "error", ex );
		}
	}

	public void send( FIXMessage msg ) throws Exception {
		SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd-HH:mm:ss" );
		Date now = new Date();
		FIXParser parser = getSocket().getParser();
		parser.putHeader( msg, "SendingTime", sdf.format( now ));
		int seq = store.getSendSeqNum() + 1;
		parser.putHeader( msg, "MsgSeqNum", String.valueOf( seq ));
		store.setSendSeqNum( seq );
		socket.send( msg );
		log( IMessageLogger.Direction.SEND, msg );
	}

	public FIXMessage asyncSend( FIXMessage msg ) throws Exception {
		send( msg );
		return getMessage();
	}

	public FIXMessage getMessage() throws Exception {
		if ( que.isEmpty( getTimeout()) ) {
			logger.warn( this, "wait timeout" );
			return null;
		}
		FIXMessage res = que.get();
		return res;
	}

	public void log( IMessageLogger.Direction dir, FIXMessage msg )
		throws Exception
	{
		for ( int i = 0; i < vecLogger.size(); i++ ) {
			IMessageLogger logger = vecLogger.get( i );
			if ( logger.getDirection() == IMessageLogger.Direction.BOTH ||
				logger.getDirection() == dir )
			{
				logger.log( dir, msg );
			}
		}
	}

	public boolean logon() throws Exception {
		FIXMessage msg = new FIXMessage();
		getSocket().getParser().putHeader( msg, "MsgType", "A" );
		getSocket().getParser().putValue( msg, "EncryptMethod", "0" );
		getSocket().getParser().putValue( msg, "HeartBtInt", "30" );
		FIXMessage res = asyncSend( msg );
		if ( !res.getMsgType().equals( "A" )) {
			return false;
		}
		return true;
	}

	public void logout() throws Exception {
		FIXMessage msg = new FIXMessage();
		getSocket().getParser().putHeader( msg, "MsgType", "5" );
		FIXMessage res = asyncSend( msg );
	}

	public void setLogger( List<IMessageLogger> var ) {
		vecLogger.clear();
		vecLogger.addAll( var );
	}

	public String toString() {
		String ret = "FIXSession id=" + getSessionId();
		ret += " socket=[" + getSocket() + "]";
		return ret;
	}

	String	sessionId;	// セッションID
	FIXParser	parser;	// メッセージのパーサ
	FIXSocket	socket;	// ソケット
	int	recvBufSize;	// 受信バッファサイズ
	int	socketTimeout;	// Socketタイムアウト
	int	timeout;	// メッセージ受信タイムアウト
	boolean	debug;	// debugフラグ

	/**
	* セッションIDを設定します。
	* @param	var		設定するデータ
	*/
	public void setSessionId( String var ) {
		this.sessionId = var;
	}
	/**
	* セッションIDを取得します。
	* @return	セッションID
	*/
	public String getSessionId() {
		return sessionId;
	}

	/**
	* メッセージのパーサを設定します。
	* @param	var		設定するデータ
	*/
	public void setParser( FIXParser var ) {
		this.parser = var;
	}
	/**
	* メッセージのパーサを取得します。
	* @return	メッセージのパーサ
	*/
	public FIXParser getParser() {
		return parser;
	}

	/**
	* ソケットを設定します。
	* @param	var		設定するデータ
	*/
	public void setSocket( FIXSocket var ) {
		this.socket = var;
	}
	/**
	* ソケットを取得します。
	* @return	ソケット
	*/
	public FIXSocket getSocket() {
		return socket;
	}

	/**
	* 受信バッファサイズを設定します。
	* @param	var		設定するデータ
	*/
	public void setRecvBufSize( int var ) {
		this.recvBufSize = var;
	}
	/**
	* 受信バッファサイズを取得します。
	* @return	受信バッファサイズ
	*/
	public int getRecvBufSize() {
		return recvBufSize;
	}

	/**
	* Socketタイムアウトを設定します。
	* @param	var		設定するデータ
	*/
	public void setSocketTimeout( int var ) {
		this.socketTimeout = var;
	}
	/**
	* Socketタイムアウトを取得します。
	* @return	Socketタイムアウト
	*/
	public int getSocketTimeout() {
		return socketTimeout;
	}

	/**
	* メッセージ受信タイムアウトを設定します。
	* @param	var		設定するデータ
	*/
	public void setTimeout( int var ) {
		this.timeout = var;
	}
	/**
	* メッセージ受信タイムアウトを取得します。
	* @return	メッセージ受信タイムアウト
	*/
	public int getTimeout() {
		return timeout;
	}

	/**
	* debugフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* debugフラグを取得します。
	* @return	debugフラグ
	*/
	public boolean isDebug() {
		return debug;
	}

}
