package	jp.co.sds_corp.fixconnect.transport;

import jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;

public class Initiator extends AbstractTransporter
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	IConfiguration config;

	public Initiator() {
	}

	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		this.config = config;
		IConfiguration attr = config.getAttributes();
		host = attr.get( "host" );
		String var;
		var = attr.get( "port" );
		port = Integer.valueOf( var ).intValue();
	}

	public void start() {
		try {
			logger.debug( this, "connect to host=[" + host + "]"
				+ " port=[" + port + "]" );
			session = new FIXSession( host, port );
			session.init( config );
			session.addMessageReceiver( this );
			session.addEventListener( this );
			session.start();
		} catch ( Exception ex ) {
			logger.error( this, "connect error", ex );
		}
	}

	public void stop() {
		try {
			close();
		} catch ( Exception ex ) {
			logger.error( this, "close error", ex );
		}
	}

	public void close() throws Exception {
		session.close();
	}

	public boolean logon() throws Exception {
		return session.logon();
	}

	public void logout() throws Exception {
		session.logout();
	}

	public void send( FIXMessage msg ) throws Exception {
		session.send( msg );
	}

	String	host;	// ホスト名
	int	port;	// ポート番号
	FIXSession	session;	// session

	/**
	* ホスト名を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var;
	}
	/**
	* ホスト名を取得します。
	* @return	ホスト名
	*/
	public String getHost() {
		return host;
	}

	/**
	* ポート番号を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var;
	}
	/**
	* ポート番号を取得します。
	* @return	ポート番号
	*/
	public int getPort() {
		return port;
	}

	/**
	* sessionを設定します。
	* @param	var		設定するデータ
	*/
	public void setSession( FIXSession var ) {
		this.session = var;
	}
	/**
	* sessionを取得します。
	* @return	session
	*/
	public FIXSession getSession() {
		return session;
	}

}
